/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    timeVaryingNonuniformFixedValueFvPatchField

Description

SourceFiles
    timeVaryingNonuniformFixedValueFvPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef timeVaryingNonuniformFixedValueFvPatchField_H
#define timeVaryingNonuniformFixedValueFvPatchField_H

#include "Time.H"
#include "fixedValueFvPatchFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class graph;

/*---------------------------------------------------------------------------*\
              Class timeVaryingNonuniformFixedValueFvPatch Declaration
\*---------------------------------------------------------------------------*/

template<class Type>
class timeVaryingNonuniformFixedValueFvPatchField
:
    public fixedValueFvPatchField<Type>
{
    // Private data

  struct TimeData
  {
    fileName fileName_;
    scalar time_;

    TimeData()
    {
      fileName_="";
      time_=0.0;
    }

    bool operator<(const TimeData& t2) const
    {
      return time_<t2.time_;
    };

  };

  struct TimeDataCache
    : TimeData
  {
    autoPtr< List<Type> > fieldPtr_;

    TimeDataCache()
      : fieldPtr_(NULL)
    {
    }

    void operator=(const TimeData& t)
    {
      TimeData::time_=t.time_;
    }

    void operator=(autoPtr< List<Type> >& ptr)
    {
        //fieldPtr_.reset(ptr);
        fieldPtr_=ptr;
    }

    bool operator==(const TimeData& t)
    {
      return TimeData::time_==t.time_;
    }

    bool isInvalid()
    {
      return !fieldPtr_.valid();
    }
  };

  class TimeDataFileList
    : public List<TimeData> 
  {
  public:
    TimeDataFileList() : List<TimeData>() {};
    TimeDataFileList(const TimeDataFileList& t) : List<TimeData>(t) {};
    TimeDataFileList(const Time& time, word dirName)
    {
        
        fileNameList files=readDir
            (
		time.path() /
                time.constant() /
                dirName
            );
        
        this->setSize(files.size());
        
        forAll(files, I)
	{
            this->operator[](I).fileName_=
             time.path()/time.constant()/dirName/files[I];
            IStringStream is(files[I]);
            this->operator[](I).time_=readScalar(is);
	}
        
        sort(*this);
    }
  };

  TimeDataFileList timeData_;
  TimeDataCache cache_[2];

  void checkTable();


public:

    //- Runtime type information
    TypeName("timeVaryingNonuniformFixedValue");


    // Constructors

        //- Construct from patch and internal field
        timeVaryingNonuniformFixedValueFvPatchField
        (
            const fvPatch&,
            const Field<Type>&
        );

        //- Construct from patch, internal field and dictionary
        timeVaryingNonuniformFixedValueFvPatchField
        (
            const fvPatch&,
            const Field<Type>&,
            const dictionary&
        );

        //- Construct by mapping given timeVaryingNonuniformFixedValueFvPatchField
        //  onto a new patch
        timeVaryingNonuniformFixedValueFvPatchField
        (
            const timeVaryingNonuniformFixedValueFvPatchField<Type>&,
            const fvPatch&,
            const Field<Type>&,
            const fvPatchFieldMapper&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchField<Type> > clone() const
        {
            return tmp<fvPatchField<Type> >
            (
                new timeVaryingNonuniformFixedValueFvPatchField<Type>(*this)
            );
        }

        //- Construct as copy setting internal field reference
        timeVaryingNonuniformFixedValueFvPatchField
        (
            const timeVaryingNonuniformFixedValueFvPatchField<Type>&,
            const Field<Type>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchField<Type> > clone(const Field<Type>& iF) const
        {
            return tmp<fvPatchField<Type> >
            (
                new timeVaryingNonuniformFixedValueFvPatchField<Type>(*this, iF)
            );
        }


    // Member functions

        // Access
  /*
            //- Return the fluctuation scale
            const graph& timeData() const
            {
                return timeDataPtr_;
            }

            //- Return reference to the fluctuation scale to allow adjustment
            graph& timeData()
            {
                return timeDataPtr_;
            }
  */

        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        //- Write
        virtual void write(Ostream&) const;
};

  /*
template<>
void timeVaryingNonuniformFixedValueFvPatchField<scalar>::updateCoeffs();

template<>
void timeVaryingNonuniformFixedValueFvPatchField<vector>::updateCoeffs();

template<>
void timeVaryingNonuniformFixedValueFvPatchField<tensor>::updateCoeffs();

template<>
void timeVaryingNonuniformFixedValueFvPatchField<sphericalTensor>::updateCoeffs();

  */
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "timeVaryingNonuniformFixedValueFvPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
